# Configuration file
folder_de = "C:/Users/Kerwin Sneijders/Games/Age of Empires 2 DE/76561198140740017/resources/_common/scenario/"
folder_local = "C:/Users/Kerwin Sneijders/Documents/Python/AoE2ScenarioParser/resources/maps/"
folder_test = "C:/Users/Kerwin Sneijders/Documents/Python/AoE2ScenarioParser/tests/source/"

